"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.findRepresentationInBuildIn = exports.findBuildInIDRepresentation = exports.isBuildInIncluded = exports.BuildInMap = void 0;
var utils_1 = require("../../metadata/resolver/utils");
exports.BuildInMap = {
    // Class
    SWAGGER_TAGS: {
        name: 'SwaggerTags',
        properties: [{ amount: 'all', declaredAs: "argument" }]
    },
    // Class + Method
    RESPONSE_EXAMPLE: {
        name: 'ResponseExample',
        properties: [
            { type: "TYPE", declaredAs: "typeArgument" },
            { type: "PAYLOAD", declaredAs: "argument" }
        ]
    },
    RESPONSE_DESCRIPTION: {
        name: 'ResponseDescription',
        properties: [
            { type: "TYPE", declaredAs: "typeArgument" },
            { type: "STATUS_CODE", declaredAs: "argument", position: 0 },
            { type: "DESCRIPTION", declaredAs: "argument", position: 1 },
            { type: "PAYLOAD", declaredAs: "argument", position: 2 }
        ]
    },
    REQUEST_CONSUMES: {
        name: 'RequestConsumes',
        properties: [{ amount: 'all', declaredAs: "argument" }]
    },
    RESPONSE_PRODUCES: {
        name: 'ResponseProduces',
        properties: [{ amount: 'all', declaredAs: "argument" }]
    },
    SWAGGER_HIDDEN: {
        name: 'SwaggerHidden',
        properties: []
    },
    IS_INT: {
        name: 'IsInt',
        properties: []
    },
    IS_LONG: {
        name: 'IsLong',
        properties: []
    },
    IS_FlOAT: {
        name: 'IsFloat',
        properties: []
    },
    IS_DOUBLE: {
        name: 'IsDouble',
        properties: []
    },
    SERVER_FILES_PARAM: {
        name: 'RequestFileParam',
        properties: [{}]
    },
    SERVER_FILE_PARAM: {
        name: 'RequestFileParam',
        properties: [{}]
    },
};
function isBuildInIncluded(map, id) {
    if (typeof map === 'undefined' || typeof map.useBuildIn === 'undefined') {
        // Build in is always included by default.
        return true;
    }
    var useLibraryType = Object.prototype.toString.call(map.useBuildIn);
    switch (useLibraryType) {
        case '[object Boolean]':
            return map.useBuildIn;
        case '[object String]':
            return map.useBuildIn === id;
        case '[object Array]':
            return map.useBuildIn.indexOf(id) !== -1;
        case '[object Object]':
            return utils_1.hasOwnProperty(map.useBuildIn, id) && map.useBuildIn[id];
    }
    return true;
}
exports.isBuildInIncluded = isBuildInIncluded;
function findBuildInIDRepresentation(id, map) {
    if (typeof map === 'undefined') {
        return findRepresentationInBuildIn(id);
    }
    // check if library is included, if so check if library can provide representation.
    var useLibraryType = Object.prototype.toString.call(map.useBuildIn);
    switch (useLibraryType) {
        case '[object Boolean]':
            if (!map.useBuildIn) {
                return undefined;
            }
            break;
        case '[object String]':
            return map.useBuildIn === id ? findRepresentationInBuildIn(id) : undefined;
        case '[object Array]':
            return map.useBuildIn.indexOf(id) !== -1 ? findRepresentationInBuildIn(id) : undefined;
        case '[object Object]':
            // tslint:disable-next-line:forin
            for (var recordKey in map.useBuildIn) {
                if (!utils_1.hasOwnProperty(map.useBuildIn, recordKey)) {
                    continue;
                }
                var item = map.useBuildIn[recordKey];
                if (!item) {
                    continue;
                }
                return findRepresentationInBuildIn(recordKey);
            }
    }
    return findRepresentationInBuildIn(id);
}
exports.findBuildInIDRepresentation = findBuildInIDRepresentation;
function findRepresentationInBuildIn(key) {
    if (utils_1.hasOwnProperty(exports.BuildInMap, key)) {
        return exports.BuildInMap[key];
    }
    return undefined;
}
exports.findRepresentationInBuildIn = findRepresentationInBuildIn;
//# sourceMappingURL=index.js.map