"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.findLibraryIDRepresentation = exports.isLibraryIncluded = void 0;
var utils_1 = require("../../metadata/resolver/utils");
var express_1 = require("./@decorators/express");
var typescript_rest_1 = require("./typescript-rest");
function isLibraryIncluded(library, map) {
    if (typeof map === 'undefined') {
        return false;
    }
    var useLibraryType = Object.prototype.toString.call(map.useLibrary);
    switch (useLibraryType) {
        case '[object String]':
            return map.useLibrary === library;
        case '[object Array]':
            return map.useLibrary.indexOf(library) !== -1;
        case '[object Object]':
            for (var key in map.useLibrary) {
                if (key === library) {
                    return true;
                }
            }
            return false;
    }
    return false;
}
exports.isLibraryIncluded = isLibraryIncluded;
function findLibraryIDRepresentation(library, key, map) {
    if (typeof map === 'undefined') {
        return findRepresentationInConfig(library, key);
    }
    // check if library is included, if so check if library can provide representation.
    var useLibraryType = Object.prototype.toString.call(map.useLibrary);
    switch (useLibraryType) {
        case '[object String]':
            return map.useLibrary === library ? findRepresentationInConfig(library, key) : undefined;
        case '[object Array]':
            return map.useLibrary.indexOf(library) !== -1 ? findRepresentationInConfig(library, key) : undefined;
        case '[object Object]':
            for (var recordKey in map.useLibrary) {
                if (recordKey !== library) {
                    continue;
                }
                var item = map.useLibrary[recordKey];
                // check if only specific keys should be taken, or if the config provides another one.
                var libraryType = Object.prototype.toString.call(item);
                switch (libraryType) {
                    case '[object String]':
                        return item === key ? findRepresentationInConfig(library, key) : undefined;
                    case '[object Array]':
                        // just allow specific keys of library.
                        return item.indexOf(key) !== -1 ? findRepresentationInConfig(library, key) : undefined;
                    case '[object Object]':
                        return utils_1.hasOwnProperty(item, key) && item[key] ? findRepresentationInConfig(library, key) : undefined;
                }
            }
    }
    return undefined;
}
exports.findLibraryIDRepresentation = findLibraryIDRepresentation;
function findRepresentationInConfig(library, key) {
    switch (library) {
        case "typescript-rest":
            if (utils_1.hasOwnProperty(typescript_rest_1.TypescriptRestLibrary.DecoratorRepresentations, key)) {
                return typescript_rest_1.TypescriptRestLibrary.DecoratorRepresentations[key];
            }
            break;
        case "@decorators/express":
            if (utils_1.hasOwnProperty(express_1.DecoratorExpressLibrary.DecoratorRepresentations, key)) {
                return express_1.DecoratorExpressLibrary.DecoratorRepresentations[key];
            }
            break;
    }
    return undefined;
}
//# sourceMappingURL=index.js.map