"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __spreadArray = (this && this.__spreadArray) || function (to, from) {
    for (var i = 0, il = from.length, j = to.length; i < il; i++, j++)
        to[j] = from[i];
    return to;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Decorator = void 0;
var utils_1 = require("../metadata/resolver/utils");
var build_in_1 = require("./build-in");
var library_1 = require("./library");
var utils_2 = require("./utils");
var Decorator;
(function (Decorator) {
    var RepresentationResolver = /** @class */ (function () {
        function RepresentationResolver(id, representationConfig) {
            this.id = id;
            this.representationConfig = [];
            this.representationConfig = Array.isArray(representationConfig) ? representationConfig : [representationConfig];
        }
        // -------------------------------------------
        RepresentationResolver.prototype.getNames = function () {
            return this.representationConfig.map(function (config) { return config.name; });
        };
        // -------------------------------------------
        RepresentationResolver.prototype.isPresentOnNode = function (node) {
            var representationConfig = this.buildRepresentationConfigFromNode(node);
            return typeof representationConfig.decorator !== 'undefined';
        };
        // -------------------------------------------
        RepresentationResolver.prototype.getDecoratorPropertyValueAsArray = function (decorator, property) {
            if (typeof property === 'undefined') {
                return undefined;
            }
            var value = this.getDecoratorPropertyValue(decorator, property);
            if (typeof value === 'undefined') {
                return [];
            }
            return property.amount === 'all' ? value : [value];
        };
        RepresentationResolver.prototype.getDecoratorPropertyValueAsItem = function (decorator, property) {
            if (typeof property === 'undefined') {
                return undefined;
            }
            var value = this.getDecoratorPropertyValue(decorator, property);
            if (property.amount === 'all') {
                if (Array.isArray(value) && value.length > 0) {
                    return value[0];
                }
            }
            return value;
        };
        RepresentationResolver.prototype.getDecoratorPropertyValue = function (decorator, property) {
            if (typeof property === 'undefined') {
                return undefined;
            }
            switch (property.amount) {
                case "all":
                    switch (property.declaredAs) {
                        case 'typeArgument':
                            return decorator.typeArguments;
                        case 'argument':
                            return decorator.arguments;
                    }
                    break;
                case "one":
                    switch (property.declaredAs) {
                        case 'typeArgument':
                            if (decorator.typeArguments.length > property.position) {
                                return decorator.typeArguments[property.position];
                            }
                            break;
                        case 'argument':
                            if (decorator.arguments.length > property.position) {
                                return decorator.arguments[property.position];
                            }
                            break;
                    }
                    break;
            }
            return undefined;
        };
        // -------------------------------------------
        RepresentationResolver.prototype.getPropertyByType = function (representationName, type) {
            if (type === void 0) { type = 'SIMPLE'; }
            var args = this.getPropertiesByTypes(representationName, [type]);
            return args[type] || undefined;
        };
        RepresentationResolver.prototype.getPropertiesByTypes = function (representationName, types) {
            var properties = {};
            types.map(function (type) { return properties[type] = undefined; });
            var index = this.representationConfig.findIndex(function (config) { return config.name === representationName; });
            if (index === -1) {
                return properties;
            }
            var representation = this.representationConfig[index];
            var args = representation.properties.filter(function (arg) { return typeof arg.type === 'undefined' || types.indexOf(arg.type) !== -1; });
            for (var j = 0; j < args.length; j++) {
                var property = Decorator.RepresentationResolver.fillProperty(args[j]);
                properties[property.type] = property;
            }
            return properties;
        };
        // -------------------------------------------
        RepresentationResolver.prototype.buildRepresentationConfigFromNode = function (node) {
            var decorators = utils_2.getDecorators(node);
            var decoratorNames = decorators.map(function (decorator) { return decorator.text; });
            var _loop_1 = function (i) {
                var index = decoratorNames.indexOf(this_1.representationConfig[i].name);
                if (index !== -1) {
                    return { value: __assign({ decorator: decorators[index] || undefined, decorators: decorators.filter(function (decorator) { return decorator.text === decorators[index].text; }) }, this_1.representationConfig[i]) };
                }
            };
            var this_1 = this;
            for (var i = 0; i < this.representationConfig.length; i++) {
                var state_1 = _loop_1(i);
                if (typeof state_1 === "object")
                    return state_1.value;
            }
            return {
                decorators: [],
                decorator: undefined,
                name: undefined
            };
        };
        // -------------------------------------------
        RepresentationResolver.fillProperty = function (property) {
            if (typeof property.type === 'undefined') {
                property.type = 'SIMPLE';
            }
            if (typeof property.declaredAs === 'undefined') {
                property.declaredAs = 'argument';
            }
            if (typeof property.amount === 'undefined') {
                property.amount = 'one';
            }
            if (typeof property.position === 'undefined') {
                property.position = 0;
            }
            return property;
        };
        return RepresentationResolver;
    }());
    Decorator.RepresentationResolver = RepresentationResolver;
    // -------------------------------------------
    var handlerCache = {};
    function toManyRepresentation(representation) {
        return Array.isArray(representation) ? representation : [representation];
    }
    function getRepresentationHandler(id, map) {
        var value = [];
        if (utils_1.hasOwnProperty(handlerCache, id)) {
            return handlerCache[id];
        }
        if (typeof map === 'undefined' || typeof map.override === 'undefined' || !utils_1.hasOwnProperty(map.override, id)) {
            var libraries = [
                "@decorators/express",
                "typescript-rest"
            ];
            for (var i = 0; i < libraries.length; i++) {
                if (!library_1.isLibraryIncluded(libraries[i], map)) {
                    continue;
                }
                var currentValue = library_1.findLibraryIDRepresentation(libraries[i], id, map);
                if (typeof currentValue !== 'undefined') {
                    value = __spreadArray(__spreadArray([], value), toManyRepresentation(currentValue));
                }
            }
            if (build_in_1.isBuildInIncluded(map, id)) {
                var buildInValue = build_in_1.findBuildInIDRepresentation(id);
                if (typeof buildInValue !== 'undefined') {
                    value = __spreadArray(__spreadArray([], value), toManyRepresentation(buildInValue));
                }
            }
        }
        else {
            value = toManyRepresentation(map.override[id]);
        }
        var resolver = new RepresentationResolver(id, value);
        handlerCache[id] = resolver;
        return resolver;
    }
    Decorator.getRepresentationHandler = getRepresentationHandler;
})(Decorator = exports.Decorator || (exports.Decorator = {}));
//# sourceMappingURL=type.js.map